@@ -1,22 +1,26 @@
 """Tests for the mypy parser."""
 
+from __future__ import annotations
+
 import sys
 
 from pytest import skip
 
 from mypy import defaults
-from mypy.test.helpers import assert_string_arrays_equal, parse_options
-from mypy.test.data import DataDrivenTestCase, DataSuite
-from mypy.parse import parse
 from mypy.errors import CompileError
 from mypy.options import Options
+from mypy.parse import parse
+from mypy.test.data import DataDrivenTestCase, DataSuite
+from mypy.test.helpers import assert_string_arrays_equal, find_test_files, parse_options
 
 
 class ParserSuite(DataSuite):
     required_out_section = True
-    base_path = '.'
-    files = ['parse.test',
-             'parse-python2.test']
+    base_path = "."
+    files = find_test_files(pattern="parse*.test", exclude=["parse-errors.test"])
+
+    if sys.version_info < (3, 10):
+        files.remove("parse-python310.test")
 
     def run_case(self, testcase: DataDrivenTestCase) -> None:
         test_parser(testcase)
@@ -28,35 +32,38 @@ def test_parser(testcase: DataDrivenTestCase) -> None:
     The argument contains the description of the test case.
     """
     options = Options()
+    options.hide_error_codes = True
 
-    if testcase.file.endswith('python2.test'):
-        options.python_version = defaults.PYTHON2_VERSION
+    if testcase.file.endswith("python310.test"):
+        options.python_version = (3, 10)
     else:
         options.python_version = defaults.PYTHON3_VERSION
 
     try:
-        n = parse(bytes('\n'.join(testcase.input), 'ascii'),
-                  fnam='main',
-                  module='__main__',
-                  errors=None,
-                  options=options)
-        a = str(n).split('\n')
+        n = parse(
+            bytes("\n".join(testcase.input), "ascii"),
+            fnam="main",
+            module="__main__",
+            errors=None,
+            options=options,
+        )
+        a = str(n).split("\n")
     except CompileError as e:
         a = e.messages
-    assert_string_arrays_equal(testcase.output, a,
-                               'Invalid parser output ({}, line {})'.format(
-                                   testcase.file, testcase.line))
+    assert_string_arrays_equal(
+        testcase.output, a, f"Invalid parser output ({testcase.file}, line {testcase.line})"
+    )
 
 
 # The file name shown in test case output. This is displayed in error
 # messages, and must match the file name in the test case descriptions.
-INPUT_FILE_NAME = 'file'
+INPUT_FILE_NAME = "file"
 
 
 class ParseErrorSuite(DataSuite):
     required_out_section = True
-    base_path = '.'
-    files = ['parse-errors.test']
+    base_path = "."
+    files = ["parse-errors.test"]
 
     def run_case(self, testcase: DataDrivenTestCase) -> None:
         test_parse_error(testcase)
@@ -64,19 +71,21 @@ class ParseErrorSuite(DataSuite):
 
 def test_parse_error(testcase: DataDrivenTestCase) -> None:
     try:
-        options = parse_options('\n'.join(testcase.input), testcase, 0)
+        options = parse_options("\n".join(testcase.input), testcase, 0)
         if options.python_version != sys.version_info[:2]:
             skip()
         # Compile temporary file. The test file contains non-ASCII characters.
-        parse(bytes('\n'.join(testcase.input), 'utf-8'), INPUT_FILE_NAME, '__main__', None,
-              options)
-        raise AssertionError('No errors reported')
+        parse(
+            bytes("\n".join(testcase.input), "utf-8"), INPUT_FILE_NAME, "__main__", None, options
+        )
+        raise AssertionError("No errors reported")
     except CompileError as e:
         if e.module_with_blocker is not None:
-            assert e.module_with_blocker == '__main__'
+            assert e.module_with_blocker == "__main__"
         # Verify that there was a compile error and that the error messages
         # are equivalent.
         assert_string_arrays_equal(
-            testcase.output, e.messages,
-            'Invalid compiler output ({}, line {})'.format(testcase.file,
-                                                           testcase.line))
+            testcase.output,
+            e.messages,
+            f"Invalid compiler output ({testcase.file}, line {testcase.line})",
+        )
